/*  Orphy GTS acquisition library (liborphy)
 *  Copyright (c) Clment 'phneutre' Bourdarias
 *                   email: phneutre@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "orphy.h"

extern int fd;

/*
==================
OGTS_SendCommand

sends a command to Orphy
==================
*/
void OGTS_SendCommand(char *cmd)
{
  int i=0;

  if (OGTS_IsEnabled(OGTS_VERBOSE)) {
    OGTS_Printf("-> Sending command: %s",cmd);

    if (OGTS_GetMode() == OGTS_QUICKMODE && strstr(cmd,"XASC") == NULL)
      OGTS_Printf("\n");
  }

  for(i=0;i<strlen(cmd);i++)
    {
#ifdef OGTS_DEBUG
      OGTS_Printf("Sending: %c\n",cmd[i]);
#endif
      if (write(fd,&cmd[i],1) < 0) {
	OGTS_Error("Error writing on device %s: %s\n",device_name,strerror(errno));
      }
    }
}

/*
==================
OGTS_AskForEA

Requests data to be send from EA #<num>
==================
*/
int OGTS_AskForEA(int num)
{
  char *cmd;

  switch (OGTS_GetMode()) {
  case OGTS_TEXTMODE:
    cmd = calloc(7,sizeof(char));
    sprintf(cmd,"XEA %d\n",num);
    break;

  case OGTS_QUICKMODE:
    cmd = calloc (1,sizeof(char));
    *cmd=(64+num);
    break;

  default: return -1; break;
  }
  
  wait_flag=1;
  OGTS_SendCommand(cmd);

  free(cmd);

  return 1;
}

/*
==================
OGTS_ReadOneByte

Reads 1 byte of data on the serial port
Note that data must be present on serial port
(this does not wait for SIGIO event)
==================
*/
int OGTS_ReadOneByte(void)
{
  int i=0;
  char result[4]="00\n";
  char data='0';
  unsigned char udata=0;
  int result_int=-1;

  switch (OGTS_GetMode()) {
  case OGTS_TEXTMODE:
    while ((data != '\n')&&(i < 4)) {
      if (read(fd,&data,1) < 0) {
	OGTS_Error("Error reading on device %s: %s",device_name,strerror(errno));
      }
      //printf("data=%c (%d)\n",data,data);
      result[i] = data;
      i++;
    }
   
    result_int=atoi(result);
    break;
    
  case OGTS_QUICKMODE:
    if (read(fd,&udata,1) < 0) {
      OGTS_Error("Error reading on device %s: %s",device_name,strerror(errno));
    }
    
    //printf("data=%d\n",data);
    
    result_int=(int)(udata);
    break;

  default: return -1; break;
  }
  
  /* FIXME: don't know if we'll keep this here */
  wait_flag = 1;

  return result_int;
}

/*
==================
OGTS_ReadEA

reads and returns the value on EA #num
It sends the command, and waits for SIGIO to read the value
==================
*/
int OGTS_ReadEA(int num)
{
  int result;
  /*sigset_t mask, oldmask;

   sigemptyset (&mask);
   sigaddset (&mask, SIGIO);*/

  wait_flag=1;
  OGTS_AskForEA(num);

  time_i = OGTS_GetTicks();

  OGTS_Wait_for_signal();

  /*sigprocmask (SIG_BLOCK, &mask, &oldmask);

#ifdef OGTS_DEBUG
  OGTS_Printf("(in ReadEA) waiting for signal\n");
#endif

  while (wait_flag == 1) {

    sigsuspend (&oldmask);
#ifdef OGTS_DEBUG
    OGTS_Printf("(in ReadEA) we recieved a signal\n");
#endif

  }

  sigprocmask (SIG_UNBLOCK, &mask, NULL);

#ifdef OGTS_DEBUG
  OGTS_Printf("(in ReadEA) signal catched\n");
#endif

*/
  result = OGTS_ReadOneByte();

  return result;
}



/*
==================
OGTS_AskForEF

Requests data to be send from EA #<num>
==================
*/
int OGTS_AskForEF(int num)
{
  char *cmd;

  switch (OGTS_GetMode()) {
  case OGTS_TEXTMODE:
    cmd = calloc(8,sizeof(char));
    sprintf(cmd,"XCPT %d\n",num);
    break;

  case OGTS_QUICKMODE:
    cmd = calloc (1,sizeof(char));
    *cmd=(208+num);
    break;

  default: return -1; break;
  }
  
  wait_flag=1;
  OGTS_SendCommand(cmd);

  free(cmd);
  return 1;
}

/*
==================
OGTS_ReadEF

reads and returns the value on EF #num
It sends the command, and waits for SIGIO to read the value
==================
*/
int OGTS_ReadEF(int num)
{
  int result;

  wait_flag=1;
  OGTS_AskForEF(num);

  time_i = OGTS_GetTicks();

  OGTS_Wait_for_signal();

  /*  while (wait_flag == 1) {
    pause();
    }*/

  result = OGTS_ReadOneByte();

  return result;
}

